
#include "BufferTest.hpp"

#include <cstring>
#include <cppunit/config/SourcePrefix.h>
#include <cppunit/TestAssert.h>
#include "buffer.hpp"

using namespace std;

CPPUNIT_TEST_SUITE_REGISTRATION(BufferTest);

#define BSIZE(b) ((int) (sizeof b / sizeof (int)))
#define BBYTES(b, delta) (sizeof (int) * (BSIZE(b) + (delta)))

const int BufferTest::d1[] = {10, 11, 12, 13, 14, 15, 16, 17};
const int BufferTest::d2[] = {20, 21, 22, 23, 24, 25, 26, 27};
//const int BufferTest::d3[] = {30, 31, 32, 33, 34, 35, 36, 37};

const int BufferTest::i1[] = {1, 3, 6};
const int BufferTest::i2[] = {0, 1, 2, 3, 4, 5, 6, 7};
const int BufferTest::i3[] = {1, 2, 5};
const int BufferTest::i4[] = {0, 4, 5, 7};
const int BufferTest::i5[] = {0, 1, 3, 4, 6};

const int BufferTest::r1[] = {10, 12, 14, 15, 17}; // d1 remove i1.
const int BufferTest::r2[] = {10, 20, 21, 11, 12, 22, 13, 14, 15, 16, 17}; // d1 insert i3,d2.
const int BufferTest::r3[] = {10, 11, 12, 14, 15, 20, 16, 17}; // d1 remove [3] insert [5],d2.
const int BufferTest::r4[] = {20, 21, 11, 22, 23, 12, 24, 13, 16}; // d1 remove i4 insert i5,d2.

void BufferTest::setUp() {
	memset(b1, 0, sizeof b1);
	memset(b2, 0, sizeof b2);
	//memset(b3, 0, sizeof b3);
}

void BufferTest::tearDown() {
	
}


void BufferTest::testRemoveNone() {
	memcpy(b1, d1, sizeof d1);
	
	CPPUNIT_ASSERT_EQUAL(
		buffer::remove(BSIZE(d1), b1, 0, NULL),
		BSIZE(d1));
	
	CPPUNIT_ASSERT(!memcmp(b1, d1, sizeof d1));
}

void BufferTest::testRemoveFirst() {
	memcpy(b1, d1, sizeof d1);
	
	CPPUNIT_ASSERT_EQUAL(
		buffer::remove(BSIZE(d1), b1, 0),
		BSIZE(d1)-1);
	
	CPPUNIT_ASSERT(!memcmp(b1, d1 + 1, BBYTES(d1, -1)));
}

void BufferTest::testRemoveLast() {
	memcpy(b1, d1, sizeof d1);
	
	CPPUNIT_ASSERT_EQUAL(
		buffer::remove(BSIZE(d1), b1, BSIZE(d1)-1),
		BSIZE(d1)-1);
	
	CPPUNIT_ASSERT(!memcmp(b1, d1, BBYTES(d1, -1)));
}

void BufferTest::testRemoveN() {
	memcpy(b1, d1, sizeof d1);
	
	CPPUNIT_ASSERT_EQUAL(
		buffer::remove(BSIZE(d1), b1, BSIZE(i1), i1),
		BSIZE(r1));
	
	CPPUNIT_ASSERT(!memcmp(b1, r1, sizeof r1));
}

void BufferTest::testRemoveAll() {
	memcpy(b1, d1, sizeof d1);
	
	CPPUNIT_ASSERT_EQUAL(
		buffer::remove(BSIZE(d1), b1, BSIZE(i2), i2),
		0);
}


void BufferTest::testInsertNone() {
	memcpy(b1, d1, sizeof d1);
	
	CPPUNIT_ASSERT_EQUAL(
		buffer::insert(BSIZE(d1), b1, 0, NULL, (int *) NULL),
		BSIZE(d1));
	
	CPPUNIT_ASSERT(!memcmp(b1, d1, sizeof d1));
}

void BufferTest::testInsertFirst() {
	memcpy(b1, d1, sizeof d1);
	
	memcpy(b2 + 1, d1, sizeof d1);
	b2[0] = d2[0];
	
	CPPUNIT_ASSERT_EQUAL(
		buffer::insert(BSIZE(d1), b1, 0, d2[0]),
		BSIZE(d1)+1);
	
	CPPUNIT_ASSERT(!memcmp(b1, b2, BBYTES(d1, 1)));
}

void BufferTest::testInsertLast() {
	memcpy(b1, d1, sizeof d1);
	
	memcpy(b2, d1, sizeof d1);
	b2[BSIZE(d1)] = d2[0];
	
	CPPUNIT_ASSERT_EQUAL(
		buffer::insert(BSIZE(d1), b1, BSIZE(d1), d2[0]),
		BSIZE(d1)+1);
	
	CPPUNIT_ASSERT(!memcmp(b1, b2, BBYTES(d1, 1)));
}

void BufferTest::testInsertN() {
	memcpy(b1, d1, sizeof d1);
	
	CPPUNIT_ASSERT_EQUAL(
		buffer::insert(BSIZE(d1), b1, BSIZE(i3), i3, d2),
		BSIZE(r2));
	
	CPPUNIT_ASSERT(!memcmp(b1, r2, sizeof r2));
}

void BufferTest::testInsertAll() {
	
	CPPUNIT_ASSERT_EQUAL(
		buffer::insert(0, b1, BSIZE(i2), i2, d1),
		BSIZE(d1));
	
	CPPUNIT_ASSERT(!memcmp(b1, d1, sizeof d1));
}


void BufferTest::testRearrangeNone() {
	memcpy(b1, d1, sizeof d1);
	
	CPPUNIT_ASSERT_EQUAL(
		buffer::rearrange(BSIZE(d1), b1, 0, NULL, 0, NULL, (int *) NULL),
		BSIZE(d1));
	
	CPPUNIT_ASSERT(!memcmp(b1, d1, sizeof d1));
}

void BufferTest::testRearrange1() {
	memcpy(b1, d1, sizeof d1);
	
	CPPUNIT_ASSERT_EQUAL(
		buffer::rearrange(BSIZE(d1), b1, 3, 5, d2[0]),
		BSIZE(d1));
	
	CPPUNIT_ASSERT(!memcmp(b1, r3, sizeof r3));
}

void BufferTest::testRearrangeN() {
	memcpy(b1, d1, sizeof d1);
	
	CPPUNIT_ASSERT_EQUAL(
		buffer::rearrange(BSIZE(d1), b1, BSIZE(i4), i4, BSIZE(i5), i5, d2),
		BSIZE(r4));
	
	CPPUNIT_ASSERT(!memcmp(b1, r4, sizeof r4));
}

void BufferTest::testRearrangeAll() {
	memcpy(b1, d1, sizeof d1);
	
	CPPUNIT_ASSERT_EQUAL(
		buffer::rearrange(BSIZE(d1), b1, BSIZE(i2), i2, BSIZE(i2), i2, d2),
		BSIZE(d2));
	
	CPPUNIT_ASSERT(!memcmp(b1, d2, sizeof d2));
}
