/*
Copyright (c) 2007 Johan Sarge

Permission is hereby granted, free of charge, to any person obtaining a copy of this
software and associated documentation files (the "Software"), to deal in the Software
without restriction, including without limitation the rights to use, copy, modify, merge,
publish, distribute, sublicense, and/or sell copies of the Software, and to permit
persons to whom the Software is furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all copies or
substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE
FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR
OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
DEALINGS IN THE SOFTWARE.
*/

#include "ConverterRegistry.hpp"

#include <algorithm>
#include <stdexcept>

std::vector<Converter *> ConverterRegistry::converters;

void ConverterRegistry::addConverter(Converter *c) {
	if (c->getSourceID() <= 0 || c->getTargetID() <= 0 || c->getSourceID() == c->getTargetID())
		throw new std::invalid_argument("Converter has invalid source and/or target ID.");
	
	converters.push_back(c);
}

void ConverterRegistry::removeConverter(Converter *c) {
	std::vector<Converter *>::iterator i = std::find(converters.begin(), converters.end(), c);
	if (i != converters.end())
		converters.erase(i);
}

std::vector<Converter *> &ConverterRegistry::getConverters(std::vector<Converter *> &output) {
	output = converters;
	return output;
}
