/*
Copyright (c) 2007 Johan Sarge

Permission is hereby granted, free of charge, to any person obtaining a copy of this
software and associated documentation files (the "Software"), to deal in the Software
without restriction, including without limitation the rights to use, copy, modify, merge,
publish, distribute, sublicense, and/or sell copies of the Software, and to permit
persons to whom the Software is furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all copies or
substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE
FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR
OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
DEALINGS IN THE SOFTWARE.
*/

#ifndef FIXER_CONVERTERREGISTRY_HPP
#define FIXER_CONVERTERREGISTRY_HPP

#include <vector>
#include "Converter.hpp"

#define FIXER__TOKEN_CONCAT(a, b) a ## b

#define FIXER_TOKEN_CONCAT(a, b) FIXER__TOKEN_CONCAT(a, b)

#define FIXER_MAKE_UNIQUE_NAME(prefix) FIXER_TOKEN_CONCAT(prefix, __LINE__)

#define REGISTER_CONVERTER_CLASS(ConverterClass) \
	static ConverterWrapper<ConverterClass> FIXER_MAKE_UNIQUE_NAME(autoRegisteredConverter__)

class ConverterRegistry {
public:
	static void addConverter(Converter *converter);
	
	static void removeConverter(Converter *converter);
	
	static std::vector<Converter *> &getConverters(std::vector<Converter *> &output);
	
private:
	static std::vector<Converter *> converters;
};

template <typename CT>
class ConverterWrapper {
public:
	ConverterWrapper();
	
	~ConverterWrapper();
	
private:
	CT wrapped;
};

template <typename CT>
ConverterWrapper<CT>::ConverterWrapper() : wrapped() {
	ConverterRegistry::addConverter(&wrapped);
}

template <typename CT>
ConverterWrapper<CT>::~ConverterWrapper() {
	ConverterRegistry::removeConverter(&wrapped);
}

#endif
