/*
Copyright (c) 2007 Johan Sarge

Permission is hereby granted, free of charge, to any person obtaining a copy of this
software and associated documentation files (the "Software"), to deal in the Software
without restriction, including without limitation the rights to use, copy, modify, merge,
publish, distribute, sublicense, and/or sell copies of the Software, and to permit
persons to whom the Software is furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all copies or
substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE
FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR
OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
DEALINGS IN THE SOFTWARE.
*/

#include "buffer.hpp"
#include "fixerutils.hpp"
#include "ConverterRegistry.hpp"
#include "fxp.hpp"
#include "losttech102.hpp"
#include "losttech200.hpp"

class LostTech102To200 : public Converter {
	int getSourceID() {return 1000102;}
	
	int getTargetID() {return 1000200;}
	
	const char *getSourceDescription() {return "Lost Technology v0.1.2";}
	
	const char *getTargetDescription() {return "Lost Technology v0.2";}
	
	bool isConvertable(int nBytes, const char *bytes) {
		return
			nBytes == kParamValueOffset + 4 * LT102::kNumAllParams &&
	    *((const int *) (bytes + kPlugIdOffset)) == 0x51435143 && // "CQCQ"
			*((const int *) (bytes + kDataSizeOffset)) == 0xf8000000 &&
			*((const int *) (bytes + kParamCountOffset)) == 0x32000000;
	}
	
	int convert(int nBytes, int maxBytes, char *bytes) {
		int nBytes2 = kParamValueOffset + 4 * LT200::kNumAllParams;
		if (nBytes2 > maxBytes)
			return -1;
		
		// Edit data size field.
		int *dataSizeP = (int *) (bytes + kDataSizeOffset);
		*dataSizeP = reverseByteOrder(reverseByteOrder(*dataSizeP) + 4 * 3);
		
		// Edit parameter count field.
		int *paramCountP = (int *) (bytes + kParamCountOffset);
		*paramCountP = reverseByteOrder(reverseByteOrder(*paramCountP) + 3);
		
		// Insert new parameter values.
		int
			*paramP = (int *) (bytes + kParamValueOffset),
			inserted[] = {
				LT200::getParamOffset(LT200::kInvTrig, 0),
				LT200::getParamOffset(LT200::kInvTrig, 1),
				LT200::kBufferSizeOffset
			},
			insertedValues[] = {0, 0, reverseByteOrderF(2.0f)};
		
		buffer::insert(LT102::kNumAllParams, paramP, 3, inserted, insertedValues);
		
		return nBytes2;
	}
};

REGISTER_CONVERTER_CLASS(LostTech102To200);
