/*
Copyright (c) 2007 Johan Sarge

Permission is hereby granted, free of charge, to any person obtaining a copy of this
software and associated documentation files (the "Software"), to deal in the Software
without restriction, including without limitation the rights to use, copy, modify, merge,
publish, distribute, sublicense, and/or sell copies of the Software, and to permit
persons to whom the Software is furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all copies or
substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE
FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR
OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
DEALINGS IN THE SOFTWARE.
*/

#include "fixerutils.hpp"
#include "ConverterRegistry.hpp"
#include "fxp.hpp"
#include "losttech203.hpp"

class LostTech203To205 : public Converter {
	int getSourceID() {return 1000203;}
	
	int getTargetID() {return 1000205;}
	
	const char *getSourceDescription() {return "Lost Technology v0.2.3";}
	
	const char *getTargetDescription() {return "Lost Technology v0.2.5";}
	
	bool isConvertable(int nBytes, const char *bytes) {
		return
			nBytes == kParamValueOffset + 4 * LT203::kNumAllParams &&
			*((const int *) (bytes + kPlugIdOffset)) == 0x51435143 && // "CQCQ"
			*((const int *) (bytes + kParamValueOffset)) == reverseByteOrder(0x3f000000 | 203);
	}
	
	/**
		This converter updates the values of modulator function parameters
		so that presets that were made before the Fine and Conv functions were
		introduced are loaded with the correct modulator functions selected
		in LT 0.2.5.
	*/
	int convert(int nBytes, int maxBytes, char *bytes) {
		// Replace altered parameter values.
		int *paramP = (int *) (bytes + kParamValueOffset);
		
		*paramP = reverseByteOrder(0x3f000000 | 205); // Set plugin version field.
		
		int insertedU = 3, insertedF = 6; // Positions of the Fine and Conv functions.
		requantize(7, paramP + LT203::getParamOffset(LT203::kAModType, 0), 0, NULL, 1, &insertedU);
		requantize(7, paramP + LT203::getParamOffset(LT203::kFModType, 0), 0, NULL, 1, &insertedU);
		requantize(7, paramP + LT203::getParamOffset(LT203::kWModType, 0), 0, NULL, 1, &insertedF);
		requantize(7, paramP + LT203::getParamOffset(LT203::kAModType, 1), 0, NULL, 1, &insertedU);
		requantize(7, paramP + LT203::getParamOffset(LT203::kFModType, 1), 0, NULL, 1, &insertedU);
		requantize(7, paramP + LT203::getParamOffset(LT203::kWModType, 1), 0, NULL, 1, &insertedF);
		
		return nBytes;
	}
};

REGISTER_CONVERTER_CLASS(LostTech203To205);
