/*
Copyright (c) 2007 Johan Sarge

Permission is hereby granted, free of charge, to any person obtaining a copy of this
software and associated documentation files (the "Software"), to deal in the Software
without restriction, including without limitation the rights to use, copy, modify, merge,
publish, distribute, sublicense, and/or sell copies of the Software, and to permit
persons to whom the Software is furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all copies or
substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE
FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR
OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
DEALINGS IN THE SOFTWARE.
*/

#include "buffer.hpp"
#include "fixerutils.hpp"
#include "ConverterRegistry.hpp"
#include "fxp.hpp"

namespace R500 {
	enum { // Parameter indexes.
		kZeroX, // 0
		kInGain,
		kFuncMix,
		kAmpMin,
		kFreqMin,
		kFreqDif,
		
		kNumParams // 6
	};
}

namespace R501 {
	enum { // Parameter indexes.
		kZeroX, // 0
		kInGain,
		kSlewLim,
		kFuncMix,
		kAmpMin,
		kFreqMin,
		kFreqDif,
		kOutGain,
		
		kNumParams // 8
	};
}

class Refuzznik500To501 : public Converter {
	int getSourceID() {return 2000500;}
	
	int getTargetID() {return 2000501;}
	
	const char *getSourceDescription() {return "Refuzznik v0.5";}
	
	const char *getTargetDescription() {return "Refuzznik v0.5.1";}
	
	bool isConvertable(int nBytes, const char *bytes) {
		return
			nBytes == kParamValueOffset + 4 * R500::kNumParams &&
	    *((const int *) (bytes + kPlugIdOffset)) == 0x6e7a6652 && // "Rfzn"
			*((const int *) (bytes + kDataSizeOffset)) == 0x48000000 &&
			*((const int *) (bytes + kParamCountOffset)) == 0x06000000;
	}
	
	int convert(int nBytes, int maxBytes, char *bytes) {
		int nBytes2 = kParamValueOffset + 4 * R501::kNumParams;
		if (nBytes2 > maxBytes)
			return -1;
		
		// Edit data size field.
		int *dataSizeP = (int *) (bytes + kDataSizeOffset);
		*dataSizeP = reverseByteOrder(reverseByteOrder(*dataSizeP) + 4 * 2);
		
		// Edit parameter count field.
		int *paramCountP = (int *) (bytes + kParamCountOffset);
		*paramCountP = reverseByteOrder(reverseByteOrder(*paramCountP) + 2);
		
		// Insert new parameter values.
		int
			*paramP = (int *) (bytes + kParamValueOffset),
			inserted[] = {R501::kSlewLim, R501::kOutGain},
			insertedValues[] = {reverseByteOrderF(1.0f), reverseByteOrderF(1.0f)};
		
		buffer::insert(R500::kNumParams, paramP, 2, inserted, insertedValues);
		
		return nBytes2;
	}
};

REGISTER_CONVERTER_CLASS(Refuzznik500To501);
