#ifndef FIXER_UTILTEST_HPP
#define FIXER_UTILTEST_HPP

#include <cppunit/extensions/HelperMacros.h>

class UtilTest : public CppUnit::TestFixture {
	
	CPPUNIT_TEST_SUITE(UtilTest);
	
	CPPUNIT_TEST(testDoNothing);
	
	CPPUNIT_TEST(testRemoveFirst);
	CPPUNIT_TEST(testRemoveLast);
	CPPUNIT_TEST(testRemoveN);
	CPPUNIT_TEST(testRemoveAllButOne);
	
	CPPUNIT_TEST(testInsertFirst);
	CPPUNIT_TEST(testInsertLast);
	CPPUNIT_TEST(testInsertN);
	CPPUNIT_TEST(testInsertAllButOne);
	
	CPPUNIT_TEST(testRearrange1);
	CPPUNIT_TEST(testRearrangeN);
	CPPUNIT_TEST(testRearrangeAllButOne);
	
	CPPUNIT_TEST_SUITE_END();
	
private:
	static const int b[];
	
public:
	void setUp();
	void tearDown();
	
	void testDoNothing();
	
	void testRemoveFirst();
	void testRemoveLast();
	void testRemoveN();
	void testRemoveAllButOne();
	
	void testInsertFirst();
	void testInsertLast();
	void testInsertN();
	void testInsertAllButOne();
	
	void testRearrange1();
	void testRearrangeN();
	void testRearrangeAllButOne();
	
private:
	/**
		v:    The value to convert to a step number.
		n:    Number of steps for the specified value.
		
		Returns the step number of v.
	*/
	static int getStep(float v, int n) {return (int) (0.999f * (v * n));}
	
	/**
		Tests whether the specified value is in the specified quantization step.
		
		step: Step number to test for.
		v:    The value to test.
		n:    Number of steps for the specified value.
		
		Returns true iff step == (int) (0.999f * (v * n)).
	*/
	static bool inStep(int step, float v, int n) {return step == getStep(v, n);}
	
	/**
		Returns some arbitrary value in the specified quantization step.
	*/
	static float randomValueForStep(int step, int n);
	
	/**
		Asserts that for each beforeValue, the result of requantizing an arbitrary
		value in that step with the specified parameters is in the quantization step specified
		by the corresponding afterValue.
	*/
	static void assertRequant(
		int nBefore, int nRemoved, const int *removed, int nInserted, const int *inserted,
		int nValues, const int *beforeValues, const int *afterValues);
	
};

#endif
